#!/usr/bin/perl
# j5_xml_rpc_client_convert_SBOLXML.pl
# Nathan Hillson 30-AUG-19

# *** Copyright Notice ***
#
# Copyright (c) 2010-2019, The Regents of the University of California,
# through Lawrence Berkeley National Laboratory (subject to receipt of
# any required approvals from the U.S. Dept. of Energy). All rights
# reserved.
#
# NOTICE. This software was developed under funding from the U.S. Department of Energy.
# As such, the U.S. Government has been granted for itself and others acting on its
# behalf a paid-up, nonexclusive, irrevocable, worldwide license in the Software to
# reproduce, prepare derivative works, and perform publicly and display publicly.
# Beginning five (5) years after the date permission to assert copyright is obtained
# from the U.S. Department of Energy, and subject to any subsequent five (5) year
# renewals, the U.S. Government is granted for itself and others acting on its behalf a
# paid-up, nonexclusive, irrevocable, worldwide license in the Software to reproduce,
# prepare derivative works, distribute copies to the public, perform publicly and
# display publicly, and to permit others to do so. 
#
# This program is free software; you can redistribute it and/or modify
# it under the same terms as the Perl "Artistic License"
# (http://dev.perl.org/licenses/artistic.html).
#
# *** End of Copyright Notice ***

use XML::RPC;
use MIME::Base64;
require LWP::UserAgent;

my $conversion_method;
my $encoded_to_be_converted_file;
my $input_buffer;
my $input_file_handle;
my $j5_session_id;
my $output_file_handle;
my $output_filename;
my $result;
my $ua;
my $xmlrpc;

if (@ARGV != 3) {
  die "Usage:\n$0 j5_session_id ToBeConvertedFileName ConvertGenBankToSBOLXML_or_ConvertSBOLXMLToGenBankClean_or_ConvertSBOLXMLToGenBankPreserveSBOLInformation\n";
}

$j5_session_id = $ARGV[0];

# setting the user agent timeout to a longer period of time is required to make sure that j5 has sufficient time to complete the conversion
$ua = LWP::UserAgent->new();
$ua->timeout( 86400 ); # wait up until 24 hours for j5 to complete the conversion, then bail
$xmlrpc = XML::RPC->new('https://j5.jbei.org/bin/j5_xml_rpc.pl',lwp_useragent => $ua);

# encode the to be converted file
open($input_file_handle, "$ARGV[1]") or die "$!";
while (read($input_file_handle, $input_buffer, 60*57)) {
  $encoded_to_be_converted_file.=encode_base64($input_buffer);
}
chomp $encoded_to_be_converted_file;
close($input_file_handle);

$conversion_method = $ARGV[2];

# call the j5 xml-rpc service to design the assembly
$result = $xmlrpc->call( 'ConvertSBOLXML', { 
					    j5_session_id => $j5_session_id,
					    encoded_to_be_converted_file => $encoded_to_be_converted_file,
					    conversion_method => $conversion_method,
					   }
		       );

if (defined $result->{"faultString"}) {
  print "Error: faultCode: ",$result->{"faultCode"}," faultString: ",$result->{"faultString"},"\n";
}

else {
  if (defined $result->{"error_message"}) {
    print "Error: ",$result->{"error_message"},"\n";
  }

  $output_filename = $result->{"output_filename"};
  open($output_file_handle, ">$output_filename") or die "$!";
  print $output_file_handle decode_base64($result->{"encoded_output_file"});
  close($output_file_handle);
}
